@@ -127,7 +127,8 @@ from docutils import nodes
 from sphinx.util.compat import Directive
 
 # Our own
-from IPython import Config, InteractiveShell
+from traitlets.config import Config
+from IPython import InteractiveShell
 from IPython.core.profiledir import ProfileDir
 from IPython.utils import io
 from IPython.utils.py3compat import PY3
